# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

from devtools_testutils import recorded_by_proxy
from _decorators import MessagesPreparers
from azure.communication.messages.models import MessageTemplateItem, MessageTemplate
from _shared.utils import get_http_logging_policy
from _messages_test_case import MessagesRecordedTestCase
from azure.communication.messages._shared.utils import parse_connection_str


class TestMessageTemplateClientToGetTemplates(MessagesRecordedTestCase):

    @MessagesPreparers.messages_test_decorator
    @recorded_by_proxy
    def test_get_templates(self):
        channel_id = "b045be8c-45cd-492a-b2a2-47bae7c36959"
        raised = False

        message_template_client = self.create_message_template_client()

        try:
            with message_template_client:
                message_template_item_list = message_template_client.list_templates(channel_id)
        except:
            raised = True
            raise

        assert raised is False
        assert message_template_item_list is not None



# --------------------------------------------------------------------------
#
# Copyright (c) Microsoft Corporation. All rights reserved.
#
# The MIT License (MIT)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the ""Software""), to
# deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
# sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED *AS IS*, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.
#
# --------------------------------------------------------------------------

# cSpell:ignore ests
import pytest
import os
from devtools_testutils import (
    test_proxy,
    add_header_regex_sanitizer,
    set_default_session_settings,
    add_body_key_sanitizer,
    add_oauth_response_sanitizer,
    add_general_string_sanitizer,
    add_general_regex_sanitizer,
)
from azure.communication.messages._shared.utils import parse_connection_str


# autouse=True will trigger this fixture on each pytest run, even if it's not explicitly used by a test method
@pytest.fixture(scope="session", autouse=True)
def start_proxy(test_proxy):
    set_default_session_settings()
    add_oauth_response_sanitizer()

    FAKE_CONNECTION_STRING = "endpoint=https://sanitized.unitedstates.communication.azure.com/;accesskey=fake==="
    FAKE_ENDPOINT = "sanitized.unitedstates.communication.azure.com"
    connection_str = os.environ.get("COMMUNICATION_LIVETEST_DYNAMIC_CONNECTION_STRING", FAKE_CONNECTION_STRING)
    if connection_str is not None:
        endpoint, _ = parse_connection_str(connection_str)
        resource_name = endpoint.split(".")[0]
        add_general_string_sanitizer(target=resource_name, value="sanitized")
        add_general_regex_sanitizer(regex=connection_str, value=FAKE_CONNECTION_STRING)
        add_general_regex_sanitizer(regex=endpoint, value=FAKE_ENDPOINT)

    add_general_string_sanitizer(target="8f8c29b2-c2e4-4340-bb28-3009c8a57283", value="sanitized")
    add_body_key_sanitizer(json_path="channel_registration_id", value="sanitized")
    add_body_key_sanitizer(json_path="*.channel_registration_id", value="sanitized")
    add_body_key_sanitizer(json_path="*..channel_registration_id", value="sanitized")
    add_body_key_sanitizer(json_path="to", value="sanitized")
    add_body_key_sanitizer(json_path="*.to", value="sanitized")
    add_body_key_sanitizer(json_path="*..to", value="sanitized")
    add_body_key_sanitizer(json_path="content", value="sanitized")
    add_body_key_sanitizer(json_path="*.content", value="sanitized")
    add_body_key_sanitizer(json_path="*..content", value="sanitized")
    add_body_key_sanitizer(json_path="media_uri", value="sanitized")
    add_body_key_sanitizer(json_path="*.media_uri", value="sanitized")
    add_body_key_sanitizer(json_path="*..media_uri", value="sanitized")
    add_body_key_sanitizer(json_path="id", value="sanitized")
    add_body_key_sanitizer(json_path="*.id", value="sanitized")
    add_body_key_sanitizer(json_path="*..id", value="sanitized")
    add_body_key_sanitizer(json_path="repeatability-request-id", value="sanitized")
    add_body_key_sanitizer(json_path="*.repeatability-request-id", value="sanitized")
    add_body_key_sanitizer(json_path="*..repeatability-request-id", value="sanitized")
    add_body_key_sanitizer(json_path="repeatability-first-sent", value="sanitized")
    add_body_key_sanitizer(json_path="*.repeatability-first-sent", value="sanitized")
    add_body_key_sanitizer(json_path="*..repeatability-first-sent", value="sanitized")

    add_header_regex_sanitizer(key="P3P", value="sanitized")
    add_header_regex_sanitizer(key="Set-Cookie", value="sanitized")
    add_header_regex_sanitizer(key="Date", value="sanitized")
    add_header_regex_sanitizer(key="Cookie", value="sanitized")
    add_header_regex_sanitizer(key="client-request-id", value="sanitized")
    add_header_regex_sanitizer(key="MS-CV", value="sanitized")
    add_header_regex_sanitizer(key="X-Azure-Ref", value="sanitized")
    add_header_regex_sanitizer(key="x-ms-content-sha256", value="sanitized")
    add_header_regex_sanitizer(key="x-ms-client-request-id", value="sanitized")
    add_header_regex_sanitizer(key="x-ms-date", value="sanitized")
    add_header_regex_sanitizer(key="x-ms-ests-server", value="sanitized")
    add_header_regex_sanitizer(key="x-ms-request-id", value="sanitized")
    add_header_regex_sanitizer(key="Content-Security-Policy-Report-Only", value="sanitized")
    add_header_regex_sanitizer(key="repeatability-first-sent", value="sanitized")
    add_header_regex_sanitizer(key="repeatability-request-id", value="sanitized")
    return



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

import os
from typing import List
from devtools_testutils import recorded_by_proxy
from _decorators import MessagesPreparers
from azure.core.credentials import AccessToken
from azure.communication.messages import NotificationMessagesClient
from azure.communication.messages.models import (
    TextNotificationContent,
    ImageNotificationContent,
    TemplateNotificationContent,
    MessageReceipt,
    MessageTemplate,
    MessageTemplateText,
    MessageTemplateBindings,
    MessageTemplateValue,
    WhatsAppMessageTemplateBindings,
    WhatsAppMessageTemplateBindingsComponent,
)
from _shared.utils import get_http_logging_policy
from _messages_test_case import MessagesRecordedTestCase
from azure.communication.messages._shared.utils import parse_connection_str


class TestNotificationMessageClientForText(MessagesRecordedTestCase):

    @MessagesPreparers.messages_test_decorator
    @recorded_by_proxy
    def test_text_send_message(self):
        phone_number: str = "+14254360097"
        raised = False

        text_options = TextNotificationContent(
            channel_registration_id="b045be8c-45cd-492a-b2a2-47bae7c36959",
            to=[phone_number],
            content="Thanks for your feedback Hello.",
        )

        message_response: MessageReceipt = None
        message_client: NotificationMessagesClient = self.create_notification_message_client()

        try:
            with message_client:
                message_responses = message_client.send(text_options)
                message_response = message_responses.receipts[0]
        except:
            raised = True
            raise
        assert raised is False
        assert message_response.message_id is not None
        assert message_response.to is not None

    @MessagesPreparers.messages_test_decorator
    @recorded_by_proxy
    def test_template_send_message(self):
        phone_number: str = "+14254360097"
        input_template: MessageTemplate = MessageTemplate(name="pythontest", language="en_US")
        raised = False

        message_client: NotificationMessagesClient = self.create_notification_message_client()

        template_options = TemplateNotificationContent(
            channel_registration_id="b045be8c-45cd-492a-b2a2-47bae7c36959", to=[phone_number], template=input_template
        )

        message_response: MessageReceipt = None

        try:
            with message_client:
                message_responses = message_client.send(template_options)
                message_response = message_responses.receipts[0]
        except:
            raised = True
            raise
        assert raised is False
        assert message_response.message_id is not None
        assert message_response.to is not None

    @MessagesPreparers.messages_test_decorator
    @recorded_by_proxy
    def test_template_with_parameters_send_message(self):

        phone_number: str = "+14254360097"
        parammeter1 = MessageTemplateText(name="first", text="2")

        input_template: MessageTemplate = MessageTemplate(
            name="sample_shipping_confirmation",
            language="en_US",
            template_values=[parammeter1],
            bindings=WhatsAppMessageTemplateBindings(
                body=[WhatsAppMessageTemplateBindingsComponent(ref_value="first")]
            ),
        )
        raised = False

        template_options = TemplateNotificationContent(
            channel_registration_id="b045be8c-45cd-492a-b2a2-47bae7c36959", to=[phone_number], template=input_template
        )

        message_response: MessageReceipt = None
        message_client: NotificationMessagesClient = self.create_notification_message_client()

        try:
            with message_client:
                message_responses = message_client.send(template_options)
                message_response = message_responses.receipts[0]
        except:
            raised = True
            raise
        assert raised is False
        assert message_response.message_id is not None
        assert message_response.to is not None

    @MessagesPreparers.messages_test_decorator
    @recorded_by_proxy
    def test_image_send_message(self):
        phone_number: str = "+14254360097"
        input_media_uri: str = "https://aka.ms/acsicon1"
        raised = False

        template_options = ImageNotificationContent(
            channel_registration_id="b045be8c-45cd-492a-b2a2-47bae7c36959", to=[phone_number], media_uri=input_media_uri
        )

        message_response: MessageReceipt = None
        message_client: NotificationMessagesClient = self.create_notification_message_client()

        try:
            with message_client:
                message_responses = message_client.send(template_options)
                message_response = message_responses.receipts[0]
        except:
            raised = True
            raise
        assert raised is False
        assert message_response.message_id is not None
        assert message_response.to is not None

    @MessagesPreparers.messages_test_decorator
    @recorded_by_proxy
    def test_download_media(self):
        phone_number: str = "+14254360097"
        input_media_id: str = "77af35aa-2c17-4c32-8430-53b0559eb920"
        raised = False
        message_client: NotificationMessagesClient = self.create_notification_message_client()
        try:
            with message_client:
                media_stream = message_client.download_media(input_media_id)
        except:
            raised = True
            raise
        assert raised is False
        assert media_stream is not None



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._models import AudioNotificationContent
from ._models import DocumentNotificationContent
from ._models import ImageNotificationContent
from ._models import MessageReceipt
from ._models import MessageTemplate
from ._models import MessageTemplateBindings
from ._models import MessageTemplateDocument
from ._models import MessageTemplateImage
from ._models import MessageTemplateItem
from ._models import MessageTemplateLocation
from ._models import MessageTemplateQuickAction
from ._models import MessageTemplateText
from ._models import MessageTemplateValue
from ._models import MessageTemplateVideo
from ._models import NotificationContent
from ._models import SendMessageResult
from ._models import TemplateNotificationContent
from ._models import TextNotificationContent
from ._models import VideoNotificationContent
from ._models import WhatsAppMessageTemplateBindings
from ._models import WhatsAppMessageTemplateBindingsButton
from ._models import WhatsAppMessageTemplateBindingsComponent
from ._models import WhatsAppMessageTemplateItem

from ._enums import CommunicationMessageKind
from ._enums import CommunicationMessagesChannel
from ._enums import MessageTemplateBindingsKind
from ._enums import MessageTemplateStatus
from ._enums import MessageTemplateValueKind
from ._enums import RepeatabilityResult
from ._enums import WhatsAppMessageButtonSubType
from ._patch import __all__ as _patch_all
from ._patch import *  # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "AudioNotificationContent",
    "DocumentNotificationContent",
    "ImageNotificationContent",
    "MessageReceipt",
    "MessageTemplate",
    "MessageTemplateBindings",
    "MessageTemplateDocument",
    "MessageTemplateImage",
    "MessageTemplateItem",
    "MessageTemplateLocation",
    "MessageTemplateQuickAction",
    "MessageTemplateText",
    "MessageTemplateValue",
    "MessageTemplateVideo",
    "NotificationContent",
    "SendMessageResult",
    "TemplateNotificationContent",
    "TextNotificationContent",
    "VideoNotificationContent",
    "WhatsAppMessageTemplateBindings",
    "WhatsAppMessageTemplateBindingsButton",
    "WhatsAppMessageTemplateBindingsComponent",
    "WhatsAppMessageTemplateItem",
    "CommunicationMessageKind",
    "CommunicationMessagesChannel",
    "MessageTemplateBindingsKind",
    "MessageTemplateStatus",
    "MessageTemplateValueKind",
    "RepeatabilityResult",
    "WhatsAppMessageButtonSubType",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

from enum import Enum
from azure.core import CaseInsensitiveEnumMeta


class ApiVersion(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    V2023_08_24_PREVIEW = "2023-08-24-preview"
    V2024_02_01 = "2024-02-01"
    V2024_08_30 = "2024-08-30"


DEFAULT_VERSION = ApiVersion.V2024_08_30.value



# coding: utf-8

# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

"""
FILE: get_templates_list.py

DESCRIPTION:
    This sample demonstrates fetching WhatsApp templates created in your WhatsApp Business account. The NotificationMessageClient is 
    authenticated using a connection string.
USAGE:
    python get_templates_list.py

    Set the environment variable with your own value before running the sample:
    1) COMMUNICATION_SAMPLES_CONNECTION_STRING - the connection string in your ACS resource
    2) WHATSAPP_CHANNEL_ID - Channel created in Azure portal for Advanced Messaging.
"""

import os
import sys

sys.path.append("..")


class GetTemplatesSample(object):

    connection_string = os.getenv("COMMUNICATION_SAMPLES_CONNECTION_STRING")
    channel_id = os.getenv("WHATSAPP_CHANNEL_ID")

    def get_templates_list(self):

        from azure.communication.messages import MessageTemplateClient
        from azure.communication.messages.models import TextNotificationContent

        message_template_client = MessageTemplateClient.from_connection_string(self.connection_string)

        # calling send() with whatsapp message details
        template_list = message_template_client.list_templates(self.channel_id)

        count_templates = len(list(template_list))
        print("Successfully retrieved {} templates from channel_id {}.".format(count_templates, self.channel_id))


if __name__ == "__main__":
    sample = GetTemplatesSample()
    sample.get_templates_list()



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

import os
from typing import Callable, Any

from devtools_testutils import is_live, is_live_and_not_recording, trim_kwargs_from_test_function
from azure.communication.messages._shared.utils import parse_connection_str


class MessagesPreparersAsync(object):

    @staticmethod
    def messages_test_decorator_async(func: Callable[[], object], **kwargs: Any):
        async def wrapper(self, *args, **kwargs):
            if is_live() or is_live_and_not_recording():
                self.connection_string = os.getenv("COMMUNICATION_LIVETEST_DYNAMIC_CONNECTION_STRING")
                endpoint, _ = parse_connection_str(self.connection_string)
                self.resource_name = endpoint.split(".")[0]

            else:
                self.connection_string = (
                    "endpoint=https://sanitized.unitedstates.communication.azure.com/;accesskey=fake==="
                )
                self.resource_name = "sanitized"

            return await func(self, *args, **kwargs)

        return wrapper

    @staticmethod
    def messages_test_decorator_for_token_async(func: Callable[[], object], **kwargs: Any):
        async def wrapper(self, *args, **kwargs):
            breakpoint
            if is_live() or is_live_and_not_recording():
                self.endpoint_str = os.getenv("COMMUNICATION_LIVETEST_DYNAMIC_ENDPOINT")
            else:
                self.endpoint = "sanitized.unitedstates.communication.azure.com"

            return await func(self, *args, **kwargs)

        return wrapper



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

from abc import abstractmethod
from retry import retry
import warnings
from _shared.utils import get_http_logging_policy
from azure.communication.messages import (
    NotificationMessagesClient,
    MessageTemplateClient,
)
from devtools_testutils import AzureRecordedTestCase


class MessagesRecordedTestCase(AzureRecordedTestCase):

    def create_notification_message_client(self) -> NotificationMessagesClient:
        return NotificationMessagesClient.from_connection_string(
            conn_str=self.connection_string, http_logging_policy=get_http_logging_policy()
        )

    def create_notification_message_client_from_token(self) -> NotificationMessagesClient:
        return NotificationMessagesClient.from_token_credentials(
            endpoint=self.endpoint_str, http_logging_policy=get_http_logging_policy()
        )

    def create_message_template_client(self) -> MessageTemplateClient:
        return MessageTemplateClient.from_connection_string(
            conn_str=self.connection_string, http_logging_policy=get_http_logging_policy()
        )

    def create_message_template_client_from_token(self) -> MessageTemplateClient:
        return MessageTemplateClient.from_token_credentials(
            endpoint=self.endpoint_str, http_logging_policy=get_http_logging_policy()
        )



# pylint: disable=too-many-lines,too-many-statements
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from io import IOBase
import json
import sys
from typing import Any, AsyncIterable, AsyncIterator, Callable, Dict, IO, List, Optional, Type, TypeVar, Union, overload
import urllib.parse

from azure.core.async_paging import AsyncItemPaged, AsyncList
from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    StreamClosedError,
    StreamConsumedError,
    map_error,
)
from azure.core.pipeline import PipelineResponse
from azure.core.rest import AsyncHttpResponse, HttpRequest
from azure.core.tracing.decorator import distributed_trace
from azure.core.tracing.decorator_async import distributed_trace_async
from azure.core.utils import case_insensitive_dict

from ... import models as _models
from ..._model_base import SdkJSONEncoder, _deserialize
from ..._operations._operations import (
    build_message_template_list_templates_request,
    build_notification_messages_download_media_request,
    build_notification_messages_send_request,
)
from .._vendor import MessageTemplateClientMixinABC, NotificationMessagesClientMixinABC

if sys.version_info >= (3, 9):
    from collections.abc import MutableMapping
else:
    from typing import MutableMapping  # type: ignore  # pylint: disable=ungrouped-imports
JSON = MutableMapping[str, Any]  # pylint: disable=unsubscriptable-object
T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, Dict[str, Any]], Any]]


class NotificationMessagesClientOperationsMixin(NotificationMessagesClientMixinABC):  # pylint: disable=name-too-long

    @overload
    async def send(
        self, body: _models.NotificationContent, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.SendMessageResult:
        """Sends a notification message from Business to User.

        :param body: Details of the message to send. Required.
        :type body: ~azure.communication.messages.models.NotificationContent
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: SendMessageResult. The SendMessageResult is compatible with MutableMapping
        :rtype: ~azure.communication.messages.models.SendMessageResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def send(
        self, body: JSON, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.SendMessageResult:
        """Sends a notification message from Business to User.

        :param body: Details of the message to send. Required.
        :type body: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: SendMessageResult. The SendMessageResult is compatible with MutableMapping
        :rtype: ~azure.communication.messages.models.SendMessageResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def send(
        self, body: IO[bytes], *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.SendMessageResult:
        """Sends a notification message from Business to User.

        :param body: Details of the message to send. Required.
        :type body: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: SendMessageResult. The SendMessageResult is compatible with MutableMapping
        :rtype: ~azure.communication.messages.models.SendMessageResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def send(
        self, body: Union[_models.NotificationContent, JSON, IO[bytes]], **kwargs: Any
    ) -> _models.SendMessageResult:
        """Sends a notification message from Business to User.

        :param body: Details of the message to send. Is one of the following types:
         NotificationContent, JSON, IO[bytes] Required.
        :type body: ~azure.communication.messages.models.NotificationContent or JSON or IO[bytes]
        :return: SendMessageResult. The SendMessageResult is compatible with MutableMapping
        :rtype: ~azure.communication.messages.models.SendMessageResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping[int, Type[HttpResponseError]] = {  # pylint: disable=unsubscriptable-object
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.SendMessageResult] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _content = None
        if isinstance(body, (IOBase, bytes)):
            _content = body
        else:
            _content = json.dumps(body, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_notification_messages_send_request(
            content_type=content_type,
            api_version=self._config.api_version,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [202]:
            if _stream:
                try:
                    await response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        response_headers = {}
        response_headers["Repeatability-Result"] = self._deserialize(
            "str", response.headers.get("Repeatability-Result")
        )
        response_headers["x-ms-client-request-id"] = self._deserialize(
            "str", response.headers.get("x-ms-client-request-id")
        )

        if _stream:
            deserialized = response.iter_bytes()
        else:
            deserialized = _deserialize(_models.SendMessageResult, response.json())

        if cls:
            return cls(pipeline_response, deserialized, response_headers)  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace_async
    async def download_media(self, id: str, **kwargs: Any) -> AsyncIterator[bytes]:
        """Download the Media payload from a User to Business message.

        :param id: The stream ID. Required.
        :type id: str
        :return: AsyncIterator[bytes]
        :rtype: AsyncIterator[bytes]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping[int, Type[HttpResponseError]] = {  # pylint: disable=unsubscriptable-object
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[AsyncIterator[bytes]] = kwargs.pop("cls", None)

        _request = build_notification_messages_download_media_request(
            id=id,
            api_version=self._config.api_version,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = kwargs.pop("stream", True)
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    await response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        response_headers = {}
        response_headers["x-ms-client-request-id"] = self._deserialize(
            "str", response.headers.get("x-ms-client-request-id")
        )
        response_headers["content-type"] = self._deserialize("str", response.headers.get("content-type"))

        deserialized = response.iter_bytes()

        if cls:
            return cls(pipeline_response, deserialized, response_headers)  # type: ignore

        return deserialized  # type: ignore


class MessageTemplateClientOperationsMixin(MessageTemplateClientMixinABC):

    @distributed_trace
    def list_templates(self, channel_id: str, **kwargs: Any) -> AsyncIterable["_models.MessageTemplateItem"]:
        """List all templates for given Azure Communication Services channel.

        :param channel_id: The registration ID of the channel. Required.
        :type channel_id: str
        :return: An iterator like instance of MessageTemplateItem
        :rtype:
         ~azure.core.async_paging.AsyncItemPaged[~azure.communication.messages.models.MessageTemplateItem]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        maxpagesize = kwargs.pop("maxpagesize", None)
        cls: ClsType[List[_models.MessageTemplateItem]] = kwargs.pop("cls", None)

        error_map: MutableMapping[int, Type[HttpResponseError]] = {  # pylint: disable=unsubscriptable-object
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(next_link=None):
            if not next_link:

                _request = build_message_template_list_templates_request(
                    channel_id=channel_id,
                    maxpagesize=maxpagesize,
                    api_version=self._config.api_version,
                    headers=_headers,
                    params=_params,
                )
                path_format_arguments = {
                    "endpoint": self._serialize.url(
                        "self._config.endpoint", self._config.endpoint, "str", skip_quote=True
                    ),
                }
                _request.url = self._client.format_url(_request.url, **path_format_arguments)

            else:
                # make call to next link with the client's api-version
                _parsed_next_link = urllib.parse.urlparse(next_link)
                _next_request_params = case_insensitive_dict(
                    {
                        key: [urllib.parse.quote(v) for v in value]
                        for key, value in urllib.parse.parse_qs(_parsed_next_link.query).items()
                    }
                )
                _next_request_params["api-version"] = self._config.api_version
                _request = HttpRequest(
                    "GET", urllib.parse.urljoin(next_link, _parsed_next_link.path), params=_next_request_params
                )
                path_format_arguments = {
                    "endpoint": self._serialize.url(
                        "self._config.endpoint", self._config.endpoint, "str", skip_quote=True
                    ),
                }
                _request.url = self._client.format_url(_request.url, **path_format_arguments)

            return _request

        async def extract_data(pipeline_response):
            deserialized = pipeline_response.http_response.json()
            list_of_elem = _deserialize(List[_models.MessageTemplateItem], deserialized["value"])
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.get("nextLink") or None, AsyncList(list_of_elem)

        async def get_next(next_link=None):
            _request = prepare_request(next_link)

            _stream = False
            pipeline_response: PipelineResponse = await self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
                _request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                raise HttpResponseError(response=response)

            return pipeline_response

        return AsyncItemPaged(get_next, extract_data)
